/****************************************************************************
 Module
     EF_Events.h
 Description
     header file with definitions for the event structure and enumerations
     of events
 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 11/13/11 13:19 bdg      customized for TicTacDoh
 10/23/11 22:01 jec      customized for Remote Lock problem
 10/17/11 08:47 jec      changed event prefixes to EF_ to match new framework
 08/09/06 14:30 jec      started coding
*****************************************************************************/

#ifndef EF_Events_H
#define EF_Events_H

#include "stdint.h"

// Universal events occupy the lowest entries, followed by user-defined events
typedef enum {  EF_NO_EVENT = 0,
                EF_INIT,   /* used to transition from initial pseudo-state */
                EF_NEW_KEY, /* signals a new key received from terminal */
				        EF_TIMEOUT, /* signals that the timer has expired */
                /* User-defined events start here */
                /*EF_ShakeController1Z,
                EF_ShakeController2Z,
                EF_TiltController1X,
                EF_TiltController2X,
                EF_TiltController1Y,
                EF_TiltController2Y,
                EF_RotateController1,
                EF_RotateController2, */
                EF_Button1Press,
                EF_Button2Press,
                                EF_AnalogInput,
                EF_TargetHit,
                EF_Player1Timeout,
                EF_Player2Timeout,
                EF_Player1WinsToss,
                EF_Player2WinsToss,
                EF_Player1TimeDecrement,
                EF_Player2TimeDecrement,
                EF_VictoryDanceTimeDecrement,
                EF_VictoryDanceTimeout,
                EF_WinEvent,
                EF_TieEvent
                } EF_EventTyp_t ;

typedef struct EF_Event_t {
    EF_EventTyp_t EventType;    // what kind of event?
    uint16_t   EventParam;      // parameter value for use w/ this event
}EF_Event;


#endif                                  /* EF_Events_H */
