/****************************************************************************
 Module
   U:\axwu\My Documents\ME218A Code\Lab4 Event Framework\MorseCodeDecoderSM\sources\MorseDecodeSM.c

 Revision
   1.0.1

 Description
   This is a template file for implementing state machines under the EF
   Event Framework.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 10/30/11 17:59 jec      fixed references to CurrentEvent in RunTemplateSM()
 10/23/11 18:20 jec      began conversion from SMTemplate.c (02/20/07 rev)
 11/07/11 12:00 Alice Wu adapted SMTemplate.c for MorseDecodeModule 
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "misc.h"
#include "EF_General.h"
#include "EF_Queue.h"
#include "EF_Events.h"
//#include "EF_SMTemplate.h"
#include "EF_Timers.h"
#include "EF_PostList.h"
#include "EventCheckers.h"
#include <ME218_C32.h>
#include <string.h>
#include <stdio.h>
#include "BoardScoreSM.h"
#include "TicTacDohSM.h"
#include "InitializeIO.h"


/*----------------------------- Module Defines ----------------------------*/
#define QUEUE_SIZE 3

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/
void InitBoard(void);
void ClearInputRegister(void);
//char DecodeMorse (void);
void delay(int time);
void timerDelay(unsigned int wait);
void SetPlayerTime(void);
void LEDCat(void);
void ClearLED(void);
void UpdatePlayerTime(unsigned char player);
void UpdateScore(void);
void Player1Wins(void);
void Player2Wins(void);
void ResetNumbers(void);
boolean CheckWinner(int player);
boolean BoardFull(void);

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match htat of enum in header file
static TicTacDohState_t CurrentState;
static char ShiftRegisterData[16]; //16 items, index from 0-15 - gets input from board
static char LEDOutAll[] = "0000000000000000000000000000000000000000";//[] = "0000111111111111"; 
//static char LEDOutAll[8];//[] = "0000111111111111";  
//static char LEDOutAll[] = "1111111111111111"; //16 for now to test
static char LEDFiller[] = "000";
/*in code, concatination ScoreKeeper, Player1Time, Player2Time as LEDOutAll - 34 spots for 1s, the rest zeros - 5 shift registers of 8's needed 3 LARGE LED for state
3 zeros filler*/
static char GameStateLED[] =  "000";
static char ScoreKeeper[] =   "000000000000000000"; //18 to keep all scores -- alternate greens/yellows
static char TargetHit[16];
static char Player1Score[]  = "000000000"; //9 for now
static char Player2Score[]  = "000000000"; //9 for now
static char board[] =         "000000000"; //all targets are hit
static char Player1Time[] =   "00000000";  //8 index countdown
static char Player2Time[] =   "00000000";
static char TimerLightsOn[] = "11111111";
static char zeros8[] = "00000000";
static char zeros9[] = "000000000";
static char zeros16[] = "0000000000000000";
static char zeros18[] = "000000000000000000";
static char zeros40[] = "0000000000000000000000000000000000000000";
//static char LegalChars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890?.,:'-/()\"= !$&+;@_";
/*ways to win*/
/*
static char w1[] =   "111000000";
static char w2[] =   "000111000";
static char w3[] =   "000000111";
static char w4[] =   "100100100";
static char w5[] =   "010010010";
static char w6[] =   "001001001";
static char w7[] =   "100010001";
static char w8[] =   "001010100";
static char full[] = "111111111";
*/
static unsigned char p1score = 0;
static unsigned char p2score = 0;
static unsigned char p1time = 7; //8 entries, index up to 7
static unsigned char p2time = 7;
static unsigned char winner = 0;
boolean toggle = FALSE;

//static char

// everybody needs a queue too!
EF_Event BoardScoreSMQueue[QUEUE_SIZE+1];


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitTemplateSM

 Parameters
     None

 Returns
     boolean, False if error in initialization, True otherwise

 Description
     Sets up the event queue and does any rother equired initialization for
     this state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 18:55
****************************************************************************/
boolean InitBoardScoreSM ( void )
{
  unsigned char ucResult1;
  EF_Event ThisEvent;
  
  ucResult1 = EF_InitQueue( BoardScoreSMQueue, ARRAY_SIZE(BoardScoreSMQueue) );
  if ( ucResult1 ==  QUEUE_SIZE ) // successful queue init
  {
    // put us into the Initial PseudoState
    CurrentState = InitBoardState;
    InitBoard();
    // post the initial transition event
    
    ThisEvent.EventType = EF_INIT;
    if (PostBoardScoreSM( ThisEvent ) == True)
    {
        puts("BoardScore Initialized \r\n");
        return True;
    }else
    {
        return False;
    }
    
  } else
  {
    return False;
  }
}

/****************************************************************************
 Function
     PostTemplateSM

 Parameters
     EF_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to the state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
****************************************************************************/
boolean PostBoardScoreSM( EF_Event ThisEvent )
{
  boolean bResult3;

  bResult3 = EF_EnQueueFIFO( BoardScoreSMQueue, ThisEvent );

  return bResult3;
}

/****************************************************************************
 Function
    RunTemplateSM

 Parameters
   void

 Returns
   boolean, Flase if an error encountered, True otherwise

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 10/23/11, 18:38
****************************************************************************/
boolean RunBoardScoreSM( void )
{
  EF_Event ThisEvent;
  int i;
  
  // pull an event from the queue
  ThisEvent = EF_DeQueue( BoardScoreSMQueue );
  //puts("Running Board Score");

  // we should only call the Run function if we know that there is something in
  // the queue, so if we found EF_NO_EVENT something is wrong
  if (ThisEvent.EventType == EF_NO_EVENT )
  { 

    return False;
  }else
  {
    
    CurrentState = QueryTicTacDohSM();
    //puts("Board Score Current State  Queried from TicTacDoh \n\r");

    switch (CurrentState)
    {
         /*
         Case WaitingToStart
         Turn off all lights
         If event Button1Press or Button2Press
            Turn on Coin toss light
            Turn on all timer lights
         End
         */
         case WaitingToStart:       // If current state is initial Pseudo State
            {
                  puts("BoardScore in Waiting To Start");
                  
                  //return True;
                  //Turn off all lights //reset all scores
                  ClearLED();
                  ResetNumbers();
                  
                  if ((ThisEvent.EventType == EF_Button1Press) || (ThisEvent.EventType == EF_Button2Press)) //Button1Press or Button2Press
                  {
                     
                     //Turn on Coin toss light
                     GameStateLED[0] = '0';
                     GameStateLED[1] = '1';
                     GameStateLED[2] = '0';
                     //Turn on all timer lights
                     SetPlayerTime(); 
                     LEDCat();
                     puts("BoardScore in Waiting To Start, a button was pressed, lights on. \n\r");  
                  } //end if
            } //end case
         break;
        // repeat state pattern as required for other states
         
         /*
         Case CoinToss
         If event Player1WinsToss
            Turn off coin toss light
            Turn on Player 1 turn light
         If event Player2WinsToss
            Turn off coin toss light
            Turn on Player 2 turn light

         */
         case CoinToss:
         {
            GameStateLED[0] = '0';
            GameStateLED[1] = '1';
            GameStateLED[2] = '0';
            //Turn on all timer lights
            SetPlayerTime(); 
            LEDCat();
            
            switch ( ThisEvent.EventType )
            {
               
               
                case EF_Player1WinsToss:  //green
                {
                     GameStateLED[0] = '1';
                     GameStateLED[1] = '0';
                     GameStateLED[2] = '0';
                     LEDCat();

                }
                break;
                
                case EF_Player2WinsToss:
                {
                     GameStateLED[0] = '0';
                     GameStateLED[1] = '0';
                     GameStateLED[2] = '1';
                     LEDCat();
                }
                break;
            }// end switch          
               
         } //end case
         break;
         
         /*
         Case Player1Aiming
         If event Button1Press
            Check phototransistor inputs
            If a target is hit and it is not opposing players color
                Update board state and corresponding lights
                Post event TargetHit to List 00
            End if
            If current board state indicates someone won
                Post event WinEvent to List 00
                Turn on all LEDs in winning players color
            Else if current board state full
                See which player has more squares
                Turn on all LEDs in winning players color
                Post WinEvent to List 00
            End if
         End if
         If event Player1TimeDecrement
            Turn off corresponding timer LED
         Endif
         If event Player1Timeout
            Turn on all LEDs in Player2s color
         End if
         */
         case Player1Aiming:
         {  
            GameStateLED[0] = '1';
            GameStateLED[1] = '0';
            GameStateLED[2] = '0';
            LEDCat();
            
            switch ( ThisEvent.EventType )
            {
                case EF_Button1Press: 
                {
                     //Check phototransistor inputs
                     CheckTargetInput();  //TargetHit stored in TargetHit
                     
                     //Update score - loop through TargetHit to see who scored
                     for (i = 0; i < 9; i++) 
                     {
                        //Compare target hit to Player Target
                        /*
                        If a target is hit and it is not opposing players color
                        Update board state and corresponding lights
                        Post event TargetHit to List 00
                        End if
                        */
                        if((TargetHit[i] == '1') && (Player1Score[i] == '1')) //player 1 hits own target, change target color
                        {
                           //EF_Event ThisEvent;
                           Player2Score[i] = '1';
                           Player1Score[i] = '0';
                           ++p2score;
                           --p1score;                           
                           ThisEvent.EventType = EF_TargetHit;
                           ThisEvent.EventParam = 1;
                           EF_PostList00(ThisEvent);   
                        }
                        /* 
                        if (TargetHit[i] == Player2Score[i]) //player 1 hits player 2's target, do nothing
                        {
                           Player1Score[i] = '0';   
                        }
                        */ 
                        if ( (TargetHit[i] == '1') && ('1' != Player1Score[i]) && ('1' != Player2Score[i]) ) //blank target is hit, player 1 scores
                        {
                           //EF_Event ThisEvent;
                           Player1Score[i] = '1'; //update score
                           ++p1score;
                           board[i] = '1'; //update board occupancy                          
                           ThisEvent.EventType = EF_TargetHit;
                           ThisEvent.EventParam = 1;
                           EF_PostList00(ThisEvent);   
                        }//end if
                        printf("Target: %u \n\r", TargetHit[i]-0x30);
                        printf("P1: %u \n\r", Player1Score[i]-0x30);
                        printf("P2: %u \n\r", Player2Score[i]-0x30);
                     }//end for
                     //Update Board Colors
                     UpdateScore();
                     puts("Score Updated \n\r");
                     LEDCat();
                     
                     //Check for winner
                     /*
                     If current board state indicates someone won
                     Post event WinEvent to List 00
                     Turn on all LEDs in winning players color
                     */
                     
                     if(CheckWinner(1))
                     { 
                        //update score keeper to alternating 1's for player 1
                        puts("Player 1 Won!!! \r\n");
                        //Turn on all LEDs in winning players color
                        Player1Wins();
                        winner = 1;
                        ThisEvent.EventType = EF_WinEvent;
                        ThisEvent.EventParam = 1;
                        EF_PostList00(ThisEvent);     
                     }
					      if(CheckWinner(2))
                     { 
                        //update score keeper to alternating 1's for player 2
                        puts("Player 2 Won!!! \r\n");
                        //Turn on all LEDs in winning players color
                        Player2Wins();
                        winner = 2;
                        ThisEvent.EventType = EF_WinEvent;
                        ThisEvent.EventParam = 1;
                        EF_PostList00(ThisEvent);     
                     }
                     
                     
                     /*
                     Else if current board state full
                     See which player has more squares
                     Turn on all LEDs in winning players color
                     Post WinEvent to List 00 
                     */
                     
                     if(BoardFull())
                     {
                        puts("Board Full, player with more color wins!!! \r\n");
                        ThisEvent.EventType = EF_WinEvent;
                        ThisEvent.EventParam = 1;
                        EF_PostList00(ThisEvent);   
                     }
                     
                     

                } //end case
                break;
                
                case EF_Player1Timeout:
                {
                     //player 2 wins
                     puts("Player 1 Time Out, Player 2 Won!!! \r\n");
                     Player2Wins();
                     winner = 2;
                     LEDCat();
                     ThisEvent.EventType = EF_WinEvent;
                     ThisEvent.EventParam = 1;
                     EF_PostList00(ThisEvent);
                  
                }
                break;
                
                case EF_Player1TimeDecrement:
                {
                     puts("Update Player Time, \r\n");
                     UpdatePlayerTime('1');
                     LEDCat();
                }
                break;
                
            }// end switch
                        
         } //end case
         break;
         
         
         /*
         Case Player2 Aiming
         If event Button2Press
            Check phototransistor inputs
            If a target is hit and it is not opposing players color
                Update board state and corresponding lights
                Post event TargetHit to List 00
            End if
            If current board state indicates someone won
                Post event WinEvent to List 00
                Turn on all LEDs in winning players color
            Else if current board state full
                See which player has more squares
                Turn on all LEDs in winning players color
                Post WinEvent to List 00
            End if
         End if
         If event Player2TimeDecrement
            Turn off corresponding timer LED
         Endif
         If event Player2Timeout
            Turn on all LEDs in Player1s color
         End if
         */
         case Player2Aiming:
         {
            GameStateLED[0] = '0';
            GameStateLED[1] = '0';
            GameStateLED[2] = '1';
            LEDCat();
            switch ( ThisEvent.EventType )
            {
                case EF_Button2Press: 
                {
                     
                     //Check phototransistor inputs
                     CheckTargetInput();  //TargetHit stored in TargetHit
                     //can add if needed: if no target - post button press
                                                    
                     //Update score - loop through TargetHit to see who scored
                     for (i = 0; i < 9; i++) 
                     {
                        //Compare target hit to Player Target
                        /*
                        If a target is hit and it is not opposing players color
                        Update board state and corresponding lights
                        Post event TargetHit to List 00
                        End if
                        */
                        if((TargetHit[i] == '1') && (Player2Score[i] == '1')) //player 2 hits own target, change target color
                        {
                           //EF_Event ThisEvent;
                           Player1Score[i] = '1';
                           Player2Score[i] = '0';
                           ++p1score;
                           --p2score;                           
                           ThisEvent.EventType = EF_TargetHit;
                           ThisEvent.EventParam = 1;
                           EF_PostList00(ThisEvent);
                              
                        }
                        /* 
                        if (TargetHit[i] == Player1Score[i]) //player 2 hits player 1's target, do nothing
                        {
                           Player2Score[i] = '0';   
                        }
                        */ 
                        if ( (TargetHit[i] == '1') && ('1' != Player1Score[i]) && ('1' != Player2Score[i]) ) //blank target is hit, player 2 scores
                        {
                           //EF_Event ThisEvent;
                           Player2Score[i] = '1'; //update score
                           ++p2score;
                           board[i] = '1'; //update board occupancy                          
                           ThisEvent.EventType = EF_TargetHit;
                           ThisEvent.EventParam = 1;
                           EF_PostList00(ThisEvent);   
                        }//end if
                        printf("Target: %u \n\r", TargetHit[i]-0x30);
                        printf("P1: %u \n\r", Player1Score[i]-0x30);
                        printf("P2: %u \n\r", Player2Score[i]-0x30);
                     }//end for
                     
                     //Update Board Colors
                     UpdateScore();
                     LEDCat();

                     
                     //Check for winner
                     /*
                     If current board state indicates someone won
                     Post event WinEvent to List 00
                     Turn on all LEDs in winning players color
                     */
                     if(CheckWinner(2))
                     {
                        //update score keeper to alternating 1's for player 2
                        puts("Player 2 Won!!! \r\n");
                        //Turn on all LEDs in winning players color
                        Player2Wins();
                        winner = 2;
                        ThisEvent.EventType = EF_WinEvent;
                        ThisEvent.EventParam = 1;
                        EF_PostList00(ThisEvent);     
                     }
					      if(CheckWinner(1))
                     { 
                        //update score keeper to alternating 1's for player 1
                        puts("Player 1 Won!!! \r\n");
                        //Turn on all LEDs in winning players color
                        Player1Wins();
                        winner = 1;
                        ThisEvent.EventType = EF_WinEvent;
                        ThisEvent.EventParam = 1;
                        EF_PostList00(ThisEvent);     
                     }
                     
                     
                     /*
                     Else if current board state full
                     See which player has more squares
                     Turn on all LEDs in winning players color
                     Post WinEvent to List 00
                     */
                     
                     if(BoardFull())
                     {
                        puts("Board Full, player with more color wins!!! \r\n");
                        ThisEvent.EventType = EF_WinEvent;
                        ThisEvent.EventParam = 1;
                        EF_PostList00(ThisEvent);                           
                     }
                     
                     

                } //end case
                break;
                
                case EF_Player2Timeout:
                {
                  //player 1 wins
                     puts("Player 2 Time Out, Player 1 Won!!! \r\n");
                     Player1Wins();
                     winner = 1;
                     LEDCat();
                     ThisEvent.EventType = EF_WinEvent;
                     ThisEvent.EventParam = 1;
                     EF_PostList00(ThisEvent);
                  
                }
                break;
                
                case EF_Player2TimeDecrement:
                {
                     puts("Update Player Time \r\n");
                     UpdatePlayerTime('2');
                     LEDCat();  
                }
                break; 
                
            }// end switch
         }//end case
         break;
         
         /*
         Case VictoryDance
         If event VictoryDanceTimeDecrement
            Toggle states of all LEDs in winning players color
         Else if event VictoryDanceTimeout
            Turn off all LEDs*/
         case VictoryDance:
         {
            //Update Board Colors
            switch ( ThisEvent.EventType )
            {
               case EF_VictoryDanceTimeDecrement:
               {
                  //time decrement - toggle switch - looops
                  if (toggle)  //toggle starts at FALSE, so turn everything on, set toggle to true
                  {
                     ClearLED();
                     toggle = FALSE;   
                  } 
                  else
                  {
                     if (winner == 1)
                     {
                        Player1Wins();
                        GameStateLED[0] = '1';
                        GameStateLED[1] = '0';
                        GameStateLED[2] = '0';
                     } else
                     {
                        Player2Wins();
                        GameStateLED[0] = '0';
                        GameStateLED[1] = '0';
                        GameStateLED[2] = '1';
                     }
                     LEDCat();
                     toggle = TRUE;                     
                  }
               }
               break;
               //time out - turn everything off
               case EF_VictoryDanceTimeout:
               {
                  ClearLED();
                  ResetNumbers();
                    
               }
               break;
            }
            
         }//end case
         break;
         
       
    }  // end switch on Current State
    return True;
  }  //end else
}  //end function

 /****************************************************************************
 Function
     IsTemplateSMQEmpty

 Parameters
     void

 Returns
     boolean True if the Queue is empty, False if the queue has something in it

 Description
     tests if this state machine's queue is empty
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:28
****************************************************************************/
boolean IsBoardScoreSMQEmpty( void ){

  return EF_IsQueueEmpty( BoardScoreSMQueue );
}

/****************************************************************************
 Function
     QueryTemplateSM

 Parameters
     None

 Returns
     TicTacDohState_t The current state of the Template state machine

 Description
     returns the current state of the Template state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:21
****************************************************************************/
TicTacDohState_t QueryBoardScoreSM ( void )
{
   return(CurrentState);
}

/***************************************************************************
 private functions
 ***************************************************************************/
/*void ClearMorseChar (void)
{
*/
    /*
    int i;
    for (i = 0; i < 8; i++)
    {
      MorseString[i] = '\0';
    //strcpy(MorseString, "");
    }
    */
/*
    MorseString[0] = '\0';
}
*/
/*
char DecodeMorse (void) 
{
*/
   /*
   Takes no parameters, returns either a character or a symbolic value indicating 
   failure 
 
   For every entry in the array MorseCode 
   If MorseString is the same as current position in MorseCode  
       return contents of current position in LegalChars 
   EndIf 
   EndFor 
   return ERROR, since we didn't find a matching string in MorseCode 
   */
   /*
    unsigned char i;
    //
    for (i = 0; i < strlen(LegalChars); i++)
    {
      if(strcmp(MorseString,MorseCode[i]) == 0)
      {
         //printf("LegalChar: %c\r\n", LegalChars[i]);  
         return LegalChars[i];
      } //end if      
    } //end for
    return '#';
        
}
*/

void delay(int time)
{
   int i;
   
   //This function implements a time delay
   for (i = 0; i < time; i++)
   {
      /*kill some time*/
   }
}

void timerDelay(unsigned int wait)
{
   unsigned int lastTime;
   unsigned int thisTime;
   
   //TMRS12_Init(TMRS12_RATE_1MS); //Initialize timer subsystem to run 1ms per tick
   EF_Timer_Init(EF_Timer_RATE_1MS);
   
   //start with both times = same
   lastTime = EF_Timer_GetTime();
   thisTime = lastTime;
   
   // Wait until time reaches desired delay
   while((thisTime-lastTime) < wait)
   {
      thisTime = EF_Timer_GetTime(); 
   } 
}

void InitBoard(void)
{
   //initialize some pins
   DDRM_DDRM5 = 1; //shift clock
   DDRM_DDRM2 = 1; //register clock
   DDRT_DDRT5 = 0; //input dat
   DDRM_DDRM4 = 1; //595 register clock
   DDRM_DDRM3 = 1; //595 output data
}

boolean CheckWinner(int player)
{
   
   
   /*
   static char w1[] =   "111000000";
   static char w2[] =   "000111000";
   static char w3[] =   "000000111";
   static char w4[] =   "100100100";
   static char w5[] =   "010010010";
   static char w6[] =   "001001001";
   static char w7[] =   "100010001";
   static char w8[] =   "001010100";
   */
   
   if (player == 1)
   {
      puts("Checking Winner Player 1 \n\r");

      if ( ( (Player1Score[0] == '1') && (Player1Score[1] == '1') && (Player1Score[2] == '1')) ||
           ( (Player1Score[3] == '1') && (Player1Score[4] == '1') && (Player1Score[5] == '1')) ||
           ( (Player1Score[6] == '1') && (Player1Score[7] == '1') && (Player1Score[8] == '1')) ||
           ( (Player1Score[0] == '1') && (Player1Score[3] == '1') && (Player1Score[6] == '1')) ||
           ( (Player1Score[1] == '1') && (Player1Score[4] == '1') && (Player1Score[7] == '1')) ||
           ( (Player1Score[2] == '1') && (Player1Score[5] == '1') && (Player1Score[8] == '1')) ||
           ( (Player1Score[0] == '1') && (Player1Score[4] == '1') && (Player1Score[8] == '1')) ||
           ( (Player1Score[2] == '1') && (Player1Score[4] == '1') && (Player1Score[6] == '1'))
         ) //end if
      {
         return True;   
      } else
      {
         return False;
      } 
   } 
   else if (player == 2)
   {
      puts("Checking Winner Player 2 \n\r");
      if ( ((Player2Score[0] == '1') && (Player2Score[1] == '1') && (Player2Score[2] == '1')) ||
           ((Player2Score[3] == '1') && (Player2Score[4] == '1') && (Player2Score[5] == '1')) ||
           ((Player2Score[6] == '1') && (Player2Score[7] == '1') && (Player2Score[8] == '1')) ||
           ((Player2Score[0] == '1') && (Player2Score[3] == '1') && (Player2Score[6] == '1')) ||
           ((Player2Score[1] == '1') && (Player2Score[4] == '1') && (Player2Score[7] == '1')) ||
           ((Player2Score[2] == '1') && (Player2Score[5] == '1') && (Player2Score[8] == '1')) ||
           ((Player2Score[0] == '1') && (Player2Score[4] == '1') && (Player2Score[8] == '1')) ||
           ((Player2Score[2] == '1') && (Player2Score[4] == '1') && (Player2Score[6] == '1'))
         ) //end if
      {
         return True;
      } 
      else
      {
         return False;
      }  
         
   }
   else 
   {
      return False;
   }
   
}

void SetPlayerTime(void)
{
   strncpy(Player1Time, TimerLightsOn, 8); //ascii conversion - 0x30
   strncpy(Player2Time, TimerLightsOn, 8);
}

void UpdatePlayerTime(unsigned char player) //turn off player LED one at a time
{
   if(player == '1')
   {
      Player1Time[7-p1time] = '0';  
      --p1time;
   } 
   else //player 2
   {
      Player2Time[7-p2time] = '0';
      --p2time;
   }
}

void UpdateScore(void) //put score keeper into right format for output shift registers
{
   char j;
   
   for (j = 0; j < 9; j++) //goes to 8 max - 9 entris 
   {

      /*
      if ((TargetHit[j] == '1') && (player == 1))
      {
         LEDOutAll[2*j] = '1';
         LEDOutAll[2*j+1] = '0';
      } else if ((TargetHit[j] == '1') && (player == 2))
      {
         LEDOutAll[2*j] = '0';
         LEDOutAll[2*j+1] = '1';  
      } 
      else
      {
         LEDOutAll[2*j] = '0';
         LEDOutAll[2*j+1] = '0';   
      }
      */
      
      if (Player1Score[j] == '1')
      {
         ScoreKeeper[2*j] = '1';
         ScoreKeeper[2*j+1] = '0';
      }
      if (Player2Score[j] == '1')
      {
         ScoreKeeper[2*j] = '0';
         ScoreKeeper[2*j+1] = '1';  
      }
      /*
      else
      {
         ScoreKeeper[2*j] = '0';
         ScoreKeeper[2*j+1] = '0';   
      }//end if
      */
   }//end for
} //end function

void Player1Wins(void) //set board color to player 1's
{
   char j;
   
   for (j = 0; j < 9; j++) //goes to 8 max  
   {
         ScoreKeeper[2*j] = '1';
         ScoreKeeper[2*j+1] = '0';  
   }
}

void Player2Wins(void) //set board color to player 2's 
{
   char j;
   
   for (j = 0; j < 9; j++) //goes to 8 max  
   {
         ScoreKeeper[2*j] = '0';
         ScoreKeeper[2*j+1] = '1';  
   }
}

boolean BoardFull(void)
{
   if( ((board[0] == '1') &&
       (board[1] == '1') &&
       (board[2] == '1') &&
       (board[3] == '1') &&
       (board[4] == '1') &&
       (board[5] == '1') &&
       (board[6] == '1') &&
       (board[7] == '1') &&
       (board[8] == '1'))
      )
   {
      if(p1score > p2score)
      {
         Player1Wins();
         winner = 1;
         puts("Player 1 Wins");
         return True;   
      } 
      else
      {
         Player2Wins();
         winner = 2;
         puts("Player 2 Wins"); 
         return True;
      }      
   } 
   else
   {
      return False;   
   }
}

void LEDCat(void)
{
   /*
   strcat(LEDOutAll, Player1Time);  //size of LEDOutAll at 4+9  = 13
   strcat(LEDOutAll, Player2Time);  //size of LEDOutAll at 13+9 = 22
   strcat(LEDOutAll, ScoreKeeper);  //size of LEDOutAll at 22+18 = 40 
   strcat(LEDOutAll, GameStateLED); //size of LEDOutAll at 1+3  = 4
   strcat(LEDOutAll, LEDFiller);    //size of LEDOutAll at 1
   */
   char j;
   for (j = 0; j <40; j++)
   {
      if( (j >= 0) && (j < 8) )  //goes up to 7, 7-0+1 = 8 entries
      {
         LEDOutAll[j] = Player1Time[j];
      } 
      else if ( (j >= 8) && (j < 16) ) //goes up to 15, 15-8+1 = 8 entries
      {
         LEDOutAll[j] = Player2Time[j-8];  
      } 
      else if ( (j >= 16) && (j < 34) ) //goes up to 33, 33-16+1 = 18 entries
      {
         LEDOutAll[j] = ScoreKeeper[j-16];
      } 
      else if ( (j >= 34) && (j < 37) ) //goes up to 36, 36-34+1 = 3 entries 
      {
         LEDOutAll[j] = GameStateLED[j-34];
      } 
      else //goes up to 39, 39-37+1 = 3
      {
         LEDOutAll[j] = '0'; 
      }
   }

   if (strlen(LEDOutAll) == 40)
   {
      //puts("LED Shift Out Successfully Concatinated \n\r");
         
   } else
   {
      puts("Error in LED Concatination! \n\r");
   }
   
   LEDOutput();     
}

void ClearLED(void)
{
   /*
   LEDOutAll[0] =     '\0'; //empty not full of zeros
   GameStateLED[0] =  '\0';
   GameStateLED[0] =  '\0';
   ScoreKeeper[0] =   '\0'; //18 to keep all scores
   Player1Score[0] =  '\0'; //9 for now
   Player2Score[0] =  '\0'; //9 for now
   Player1Time[0] =   '\0';  //8 index countdown
   Player2Time[0] =   '\0';


   strcat(LEDOutAll, Player1Time);  //size of LEDOutAll at 4+9  = 13
   strcat(LEDOutAll, Player2Time);  //size of LEDOutAll at 13+9 = 22
   strcat(LEDOutAll, ScoreKeeper);  //size of LEDOutAll at 22+18 = 40 
   strcat(LEDOutAll, GameStateLED); //size of LEDOutAll at 1+3  = 4
   strcat(LEDOutAll, LEDFiller);    //size of LEDOutAll at 1
   */
   strncpy(Player1Score, zeros9, 9);
   strncpy(Player2Score, zeros9, 9);
   strncpy(board, zeros9, 9);
   strncpy(Player1Time, zeros8, 8);
   strncpy(Player2Time, zeros8, 8);
   strncpy(GameStateLED, LEDFiller, 3);
   strncpy(ScoreKeeper, zeros18, 18);
   strncpy(LEDOutAll, zeros40, 40);
   LEDOutput();
}

void ResetNumbers(void)
{
   p1score = 0;
   p2score = 0;
   p1time = 7;
   p2time = 7;
   winner = 0;
   toggle = FALSE;
}

void LEDOutput(void)
{
   char j;
   for (j = 39; j >= 0; j--)   //write MSB 1st
   {
      //Read the input port associated with the DataInput bit
   	
	  //Mask the result of the read to isolate the DataInput bit
   	//If the result of the masking operation is TRUE
	   
   	if (LEDOutAll[j] == '1')
   	{
   		//Shift a 1 into the ShiftRegisterData variable
   		PTM |= BIT3HI;
   		//puts("1 \n\r");
   		   
   	}
   	//else
   	else
   	{
   		//Shift a 0 into the ShiftRegisterData variable
   	   PTM &= BIT3LO;
   	   //puts("0 \n\r");
   	   
   	}
	   //pulse (raise, then lower) the pin of the output port connected to the ShiftClock line
      /*
      PTM_PTM5 = 1;
      delay(5);
      PTM_PTM5 = 0;
      delay(5);
	   */
	   PTM |= BIT5HI;   //set bit5 high
      delay(1);        //delay(3)
      PTM &= BIT5LO;   //set bit5 low
	   delay(1);        //delay(3)
   }
   
      /*
      PTM_PTM4 = 1;
      delay(5);
      PTM_PTM4 = 0;
      delay(5);	
      */
	   PTM |= BIT4HI;   //set bit5 high
      delay(1);        //delay(3)
      PTM &= BIT4LO;   //set bit5 low
      delay(1);        //delay(3)
}


void ClearInputRegister (void)
{
    //strncpy(ShiftRegisterData, zeros16, 16);
    strncpy(TargetHit,zeros16, 16);
}

void CheckTargetInput(void)
{
   char j;
   char k;
   

   //Clear the ShiftRegisterData variable used to accumulate the incoming data
   ClearInputRegister();
   
   //Latch the input pin values into the shift register by first setting the pin of the output port connected to the Shift/Load line LOW then HIGH
   /*
         PTM_PTM2 = 0;
         delay(5);
         PTM_PTM2 = 1;
         delay(5);
   */      
   PTM &= BIT2LO;  //set bit2 low
   delay(1);       //delay(3)
   PTM |= BIT2HI;  //set bit2 high
   delay(1);       //delay(3)
  
   //Repeat 16 times:
   
   for (j = 0; j < 16; j++)
   {
      //go from hardware to software
      /*
      input j  grid  input index    target index
      1        9		0					8
      2        8		1					7
      3        7		2					6
      4	      1		3					0
      5			6		4					5
      6			3		5					2
      7			2		6					1
      8			5		7					4
      9			4		8					3
      */
      switch (j)
      {
         case 0: 
         { k = 8; }
         break;
         case 1:
         { k = 7; }
         break;
         case 2:
         { k = 6; }
         break;
         case 3:
         { k = 0; }
         break;
         case 4:
         { k = 5; }
         break;
         case 5:
         { k = 2; }
         break;
         case 6:
         { k = 1; }
         break;
         case 7:
         { k = 4; }
         break;
         case 8:
         { k = 3; }
         break;
         default:
         { k = j; }
         break;
      }// end switch      
      
      
      //Read the input port associated with the DataInput bit
   	
	   //Mask the result of the read to isolate the DataInput bit
   	//If the result of the masking operation is TRUE
	  
	  
	  
   	if ((PTT & BIT5HI) != 0) //(PTT_PTT5 == 1)
   	{
   		//Shift a 1 into the ShiftRegisterData variable
   		ShiftRegisterData[k] = 1;
   		TargetHit[k] = '1';
   			   
   	}
   	//else
   	else
   	{
   		//Shift a 0 into the ShiftRegisterData variable
   		ShiftRegisterData[k] = 0;
   		TargetHit[k] = '0';
   	}
	 //pulse (raise, then lower) the pin of the output port connected to the ShiftClock line
	   
	   PTM |= BIT5HI;   //set bit5 high
      delay(1);        //delay(3)
      PTM &= BIT5LO;   //set bit5 low
	   delay(1);        //delay(3)
	   /*
      PTM_PTM5 = 1;
      delay(5);
      PTM_PTM5 = 0;
      delay(5);
      */
   	
   }	
   //End Repeat
   //Return the value of the ShiftRegisterData variable    
}


//#define Test_BoardScore
#ifdef Test_BoardScore
void main( void) {

   int j = 0;
   static int player = 1;
   int i = 0;
   
   InitBoard();
   
   //************************************ Function Order Notes
   /*
      for every state, call LEDOutput
      -start with player timers all high
         -decrement timer with each player's aiming turn
      -start with all score = 0
      -if button hit
         -check if target is blank - if so give player 1 point
         -if target not blank (player score corresponding index) - assign point to other player in player score
      -concatinate LEDFiller, GameStateLED, ScoreKeeper, Player1Time, Player2Time  as LEDOutAll
         -strlen(LEDOutAll) == 40
      -shift out
      call LEDOutput
   */ 
   //************************************ 
   
   
   //InitializeIO(); //init some pins.
    //#define PhotoTransData  BIT5HI   //T5  INPUT
    //#define PhotoTransSC    BIT6HI   //T6  OUTPUT line 1
    //#define PhotoTransRC    BIT7HI   //T7  OUTPUT line 2
   ClearLED();   
   GameStateLED[0] = '1'; //green
   GameStateLED[1] = '0'; //red
   GameStateLED[2] = '0'; //yellow
   LEDCat();
   while (1)
   {
      //sense input
      //write output
         //code to test LEDCat + UpdateScore
         /*
         SetPlayerTime();
         Player1Score[0] = '1';
         Player1Score[2] = '1';
         Player1Score[4] = '1';
         Player1Score[6] = '1';
         Player1Score[8] = '1';
         Player2Score[1] = '1';
         Player2Score[3] = '1';
         Player2Score[5] = '1';
         Player2Score[7] = '1';
         UpdateScore();
         GameStateLED[0] = '0'; //green
         GameStateLED[1] = '1'; //red
         GameStateLED[2] = '0'; //yellow
         LEDCat();
         
         puts("Start LED Sequence \n\r");
         
         for (j = 0; j < 40; j++)
         {
            if (j == 0) 
            {
               puts("Player 1 Time \n\r");
            } else if (j == 8)
            {
               puts("Player 2 Time \n\r");
            } else if (j == 16)
            {
               puts("Scores \n\r");
            } else if (j == 34)
            {
               puts("Game State LEDs \n\r");   
            } else if (j == 37)
            {
               puts("LED fillers \n\r");   
            } else
            {
                
            }
            
            printf("%d \r\n", (LEDOutAll[j]-0x30));
         }
         */
         ClearInputRegister();
         //ClearLED();
         //Player1Score[0] =  '\0'; //9 for now
         //Player2Score[0] =  '\0'; //9 for now
         SetPlayerTime();
        /* 
         for (j = 0; j < 16; j++)
         {
            printf("Shifted: %d \r\n", ShiftRegisterData[j]);
         }
         */
         CheckTargetInput();
         
         //fake some targets

         //TargetHit[0] = '1';
         //TargetHit[2] = '1';
         
         for (j = 0; j < 9; j++)
         {
            //printf("Shifted: %d \r\n", ShiftRegisterData[j]);
            
            if ((TargetHit[j] == '1') && (player == 1))
            {
               Player1Score[j] = '1';
               board[j] = '1';
               ++p1score;
               //LEDOutAll[(2*j)] = '1';
               //LEDOutAll[(2*j+1)] = '0';
            }
            if ((TargetHit[j] == '1') && (player == 2))
            {
               Player2Score[j] = '1';
               board[j] = '1';
               ++p2score;
               //LEDOutAll[(2*j)] = '0';
               //LEDOutAll[2*j+1] = '1';  
            }
            //printf("%d \r\n", (LEDOutAll[j]-0x30));
         }
         
         UpdateScore();
         LEDCat();
         
         /*
         if(CheckWinner(1))
         {
            Player1Wins();   
         }
         if(CheckWinner(2))
         {
            Player2Wins();   
         }
         */
         if(BoardFull())
         {
            puts("Board is Full \n\r");
         }
         LEDCat();
         //LEDOutput();
         
         /*
         if(BoardFull(board))
         {
            ClearLED();
         }
         */
         
   }
   
   
      /****************************Working Code************************************/
      
      //victory dance    --- this works
         /*
         if(toggle)  //toggle starts at FALSE, so turn everything on, set toggle to true
         {
            //strncpy(LEDOutAll, zeros8, 8);
            //LEDOutput();
            ClearLED();
            timerDelay(100);
            toggle = FALSE;             
         } 
         else
         {
            for (j = 0; j < 8; j++) //goes to 8 max  
            {
                  LEDOutAll[2*j] = '0';
                  LEDOutAll[2*j+1] = '1';  
                  printf("%d", (LEDOutAll[j]-0x30));
            }
            LEDOutput();
            timerDelay(100);                     
            toggle = TRUE;
            puts("\n\r");
         }
         */
      
      //test input/output with 4 LEDs -- this works!!!
         /*
         ClearInputRegister();
         Player1Score[0] =  '\0'; //9 for now
         Player2Score[0] =  '\0'; //9 for now
         for (j = 0; j < 16; j++)
         {
            //printf("Shifted: %d \r\n", ShiftRegisterData[j]);
         }
         CheckTargetInput();
         
         //fake some targets
         //TargetHit[0] = '1';
         //TargetHit[2] = '1';
         
         for (j = 0; j < 8; j++)
         {
            //printf("Shifted: %d \r\n", ShiftRegisterData[j]);
            
            if ((TargetHit[j] == '1') && (player == 1))
            {
               Player1Score[j] = '1';
               LEDOutAll[(2*j)] = '1';
               LEDOutAll[(2*j+1)] = '0';
            }
            if ((TargetHit[j] == '1') && (player == 2))
            {
               Player2Score[j] = '1';
               LEDOutAll[(2*j)] = '0';
               LEDOutAll[2*j+1] = '1';  
            }
            //printf("%d \r\n", (LEDOutAll[j]-0x30));
         }
         
         LEDOutput();
         */
         
         //Test for UpdateScore() - this works
         /* 
         Player1Score[0] = '1';
         Player1Score[1] = '1';
         Player1Score[2] = '1';
         UpdateScore();
         for (j = 0; j < 18; j++)
         {
            printf("%d \r\n", (ScoreKeeper[j]-0x30));
         }
         */
      
	
	//Timer CountDown
      /*
      timerDelay(100);
      timerDelay(100);
      LEDOutAll[0] = '0';
      LEDOutput();
      timerDelay(100);
      timerDelay(100);
      LEDOutAll[1] = '0';
      LEDOutput();
      timerDelay(100);
      LEDOutAll[2] = '0';
      LEDOutput();
      timerDelay(100);
      LEDOutAll[3] = '0';
      LEDOutput();
  
      LEDOutput();
      for (j = 0; j < 8; j++)
      {
         timerDelay(1000);
         LEDOutAll[j] = '0';
         LEDOutput();  
      }
      */
      
      //Check Input to Grid Translation
      /* 
         ClearInputRegister();
         for (j = 0; j < 16; j++)
         {
            printf("Shifted: %d \r\n", ShiftRegisterData[j]);
         }
         CheckTargetInput();
         for (j = 0; j < 16; j++)
         {
            printf("Shifted: %d \r\n", ShiftRegisterData[j]);
            if (TargetHit[j] == '1')
            {
               printf("Target Hit: %d \r\n", j+1);   
            }
         }
      */
      
      //code to test check winner + board full
         /*
         Player1Score[0] = '1';
         Player1Score[1] = '1';
         Player1Score[2] = '1';
         if(CheckWinner(Player1Score)) 
         {
            puts("Player 1 Won!!! \r\n");
         }
         Player2Score[2] = '1';
         Player2Score[4] = '1';
         Player2Score[6] = '1';
         if(CheckWinner(Player2Score)) 
         {
            puts("Player 2 Won!!! \r\n");
         }
         
         for (j = 0; j < 9; j++)
         {
            board[j] = '1';
         }
         if(BoardFull(board))
         {
            puts("Board Full, Tied!!! \r\n");   
         }
         */   



      //Test code for string copy-ing
         //LEDOutput();
         /*
         for (j = 0; j < 8; j++)
         {
            printf("Player1Time: %d \r\n", (Player1Time[j]-0x30));
         }
         SetPlayerTime();
         for (j = 0; j < 8; j++)
         {
            printf("Player1Time: %d \r\n", (Player1Time[j]-0x30));
         }
         if (Player1Time[0] == '1')
         {
            puts("you got it!, totally black magic \r\n");
         }
         */



	   //Complete sensing configuration:
      /*
      ClearInputRegister();
         for (j = 0; j < 16; j++)
         {
            printf("Shifted: %d \r\n", ShiftRegisterData[j]);
         }
      CheckTargetInput();
         for (j = 0; j < 16; j++)
         {
            printf("Shifted: %d \r\n", ShiftRegisterData[j]);
            if(ShiftRegisterData[j] == 1)
            {
               LEDOutAll[j] = '1';
            }
            else
            {
               LEDOutAll[j] = '0';   
            }
         }
         for (j = 0; j < 16; j++)
         {
            printf("Shifted: %d \r\n", LEDOutAll[j]);
         }
      LEDOutput();
      */
      
      
      
      //Output shift register test 1
      /* 
      //LEDOutput();
      */
      
      //Input shift register test 1
      /*   
         ClearInputRegister();
         for (j = 0; j < 16; j++)
         {
            printf("Shifted: %d \r\n", ShiftRegisterData[j]);
         }
         CheckTargetInput();
         for (j = 0; j < 16; j++)
         {
            printf("Shifted: %d \r\n", ShiftRegisterData[j]);
         } 
     
      */
      
      
      /* input shift register test 2 
      while (1)
      {
         PTM_PTM2 = 0;
         delay(5);
         PTM_PTM2 = 1;
         delay(5);
         
         
         for (j = 0; j < 16; j++)
         {
            PTM_PTM5 = 1;
            delay(5);
            PTM_PTM5 = 0;
         }
         
      }
      */
}
#endif
