// Event Checking functions for use in testing the CheckUserEvents 
// functionality

#include "misc.h"
#include "EF_General.h"
#include "EF_Events.h"
#include "EF_PostList.h"
#include "EventCheckers.h"
#include "ADS12.h"
#include <stdio.h>
#include "EF_Timers.h"
#include <ME218_C32.h>

// in a real application, this would be an include for all of the state
// machine functions
#include "AnalogStuffSM.h"
//#include "BoardScoreSM.h"
//#include "TicTacDohSM.h"
#include "EF_Timers.h"

boolean CheckButton1(void);
boolean CheckButton2(void);
boolean CheckAnalog(void);

boolean CheckButton1(void)
{
	static unsigned int debounceInterval = 50; //Number of miliseconds in debounce interval
   static unsigned int lastSampleTime = 0;
   static boolean buttonState = False;
   static boolean lastButtonState = False;
   boolean returnVal = False;
   //puts("ButtonStateCheck Entered");
   if ((EF_Timer_GetTime()-lastSampleTime)>debounceInterval)
   {
      //puts("ButtonState Checked");
      buttonState = ((PTM & BIT0HI)!=0); //Return 1 if PORTM 0 is high, 0 if low
      if (buttonState != lastButtonState)
      {
         if (buttonState == 0) //Pin is low, button is down
         {
            EF_Event ThisEvent;
            ThisEvent.EventType = EF_Button1Press;
            ThisEvent.EventParam = 1;
            EF_PostList00(ThisEvent); // this could be any SM post function or EF_PostAll
            EF_PostList02(ThisEvent);
            puts("Button 1 is Down");
            returnVal = True;
         } 
         else   //Pin is high, button is up
         {
            //EF_Event ThisEvent;
            //ThisEvent.EventType = ButtonDown;
            //ThisEvent.EventParam = 1;
            //EF_PostList00(ThisEvent); // this could be any SM post function or EF_PostAll
            //returnVal = True;
         }
      }
      lastSampleTime = EF_Timer_GetTime();  //Record current time as time of last sample
      lastButtonState = buttonState;
   }
   //puts("No button events");
   return (returnVal);   //Return false if no events
}


boolean CheckButton2(void)
{
	static unsigned int debounceInterval = 50; //Number of miliseconds in debounce interval
   static unsigned int lastSampleTime = 0;
   static boolean buttonState = False;
   static boolean lastButtonState = False;
   boolean returnVal = False;
   //puts("ButtonStateCheck Entered");
   if ((EF_Timer_GetTime()-lastSampleTime)>debounceInterval)
   {
      //puts("ButtonState Checked");
      buttonState = ((PTM & BIT1HI)!=0); //Return 1 if PORTM 1 is high, 0 if low
      if (buttonState != lastButtonState)
      {
         if (buttonState == 0) //Pin is low, button is down
         {
            EF_Event ThisEvent;
            ThisEvent.EventType = EF_Button2Press;
            ThisEvent.EventParam = 1;
            EF_PostList00(ThisEvent); // this could be any SM post function or EF_PostAll
            EF_PostList02(ThisEvent);
            puts("Button 2 is Down");
            returnVal = True;
         } 
         else   //Pin is high, button is up
         {
            //EF_Event ThisEvent;
            //ThisEvent.EventType = ButtonDown;
            //ThisEvent.EventParam = 1;
            //EF_PostList00(ThisEvent); // this could be any SM post function or EF_PostAll
            //returnVal = True;
         }
      }
      lastSampleTime = EF_Timer_GetTime();  //Record current time as time of last sample
      lastButtonState = buttonState;
   }
   //puts("No button events");
   return (returnVal);   //Return false if no events
}

boolean CheckAnalog(void)
{
	static unsigned int debounceInterval = 25; //Number of miliseconds in debounce interval
   static unsigned int lastSampleTime = 0;
   boolean returnVal = False;
   if ((EF_Timer_GetTime()-lastSampleTime)>debounceInterval)
   {
      EF_Event ThisEvent;
      ThisEvent.EventType = EF_AnalogInput;
      ThisEvent.EventParam = 1;
      EF_PostList01(ThisEvent); // this could be any SM post function or EF_PostAll
      lastSampleTime = EF_Timer_GetTime();  //Record current time as time of last sample
      //puts("Analog Event");
      returnVal = True; //This function does nothing except guarantee that the
                //analogStuff state machine gets entered every cycle
   }
   //lastSampleTime = EF_Timer_GetTime();  //Record current time as time of last sample
   return returnVal;
}
