 /****************************************************************************
 Module
         ServoControl.c

 Revision
         1.0.1
         
 Description
         Code File for the Servo Control Module.

 Notes


 History
 When           Who	What/Why
 -------------- ---	--------
 11/13/11 15:50 bdg  Initial Creation

****************************************************************************/
#include "ServoControl.h"
#include "ServoLib.h"
#include "stdio.h"
#include "misc.h"

#define XLowerLimit  215
#define XUpperLimit  500
#define YLowerLimit  135
#define YUpperLimit  375
#define scaleAimX    130 //50
#define centerAimX   360 //512

#define scaleAimY    130 //50
#define centerAimY   310 //512

#define scaleSpin    50
#define centerSpin   512

#define CENTER_WIDTH 1500
#define DENOMINATOR  52

#define BOARD_SERVO_PIN 0
#define AIM_X_SERVO_PIN 1
#define AIM_Y_SERVO_PIN 2

static unsigned int ServoConvert(unsigned int sensedValue, signed int scaleValue, signed int centerValue);


/****************************************************************************
// AimGunX 
// Takes unsigned int value, which ranges from 0 - 1024, returns nothing.
// Sets the servo output to a scaled PWM signal.

// unsigned char ChannelNum  0-7 to define which bit to change
// unsigned int NewWidth     width of the desired pulse in micro-seconds 
****************************************************************************/
void AimGunX(unsigned int sensedXValue)
{
   static unsigned int pulseWidthX;
   static unsigned int LastPulseWidthX = 1500;
   unsigned int beta = 5;
   unsigned int betaMax = 10;
   
   if (sensedXValue > XUpperLimit)
   {
      sensedXValue = XUpperLimit;
   }else if (sensedXValue < XLowerLimit)
   {
      sensedXValue = XLowerLimit;
   }
   pulseWidthX = (beta*LastPulseWidthX + (betaMax-beta)*ServoConvert(sensedXValue, scaleAimX, centerAimX))/betaMax; 
   LastPulseWidthX = pulseWidthX;  
   //printf("\n\r pulseWidthX %d ", pulseWidthX);
   if(Servo12_SetPulseWidth(AIM_X_SERVO_PIN, pulseWidthX) != SERVO12_Err)
   {
      //printf("\n\r pulseWidthX %d ", pulseWidthX);
   }else
   {
      puts("ServoError");
   }
   return;
}

/****************************************************************************
// AimGunY 
// Takes unsigned int value, which ranges from 0 - 1024, returns nothing.
// Sets the servo output to a scaled PWM signal.
****************************************************************************/
void AimGunY(unsigned int sensedYValue)
{
   int flippedSensedYValue;
   static unsigned int pulseWidthY;
   static unsigned int LastPulseWidthY = 1500;
   unsigned int beta = 5;
   unsigned int betaMax = 10;
   
   flippedSensedYValue = 2 * centerAimY - (signed int)sensedYValue;
   if (flippedSensedYValue > YUpperLimit)
   {
      flippedSensedYValue = YUpperLimit;
   }else if (flippedSensedYValue < YLowerLimit)
   {
      flippedSensedYValue = YLowerLimit;
   }
   //printf("flippedY = %d", flippedSensedYValue);
   
   pulseWidthY = (beta*LastPulseWidthY + (betaMax-beta)*ServoConvert(flippedSensedYValue, scaleAimY, centerAimY))/betaMax;   
   LastPulseWidthY = pulseWidthY;
   if(Servo12_SetPulseWidth(AIM_Y_SERVO_PIN, pulseWidthY) != SERVO12_Err)
   {
      //printf("\n\r pulseWidthY %d ", pulseWidthY);
   }
   return;
}

/****************************************************************************
// SpinBoard 
// Takes unsigned int value, which ranges from 0 - 1024, returns nothing.
// Sets the servo output to a scaled PWM signal.
****************************************************************************/
void SpinBoard(unsigned int sensedPotvalue)
{
   static unsigned int pulseWidthSpin;
   static unsigned int LastPulseWidthSpin = 1500;
   unsigned int beta = 19;
   unsigned int betaMax = 20;
   /*--------filter start-----------*/
   /*--------*/
   
   pulseWidthSpin = (beta*LastPulseWidthSpin + (betaMax-beta)*ServoConvert(sensedPotvalue, scaleSpin, centerSpin))/betaMax; 
   LastPulseWidthSpin = pulseWidthSpin;  
   if(Servo12_SetPulseWidth(BOARD_SERVO_PIN, pulseWidthSpin) != SERVO12_Err)
   {
      //printf("\n\r pulseWidthSpin %d ", pulseWidthSpin);
   }
   return;
}




/****************************************************************************
// Private Functions
****************************************************************************/




/****************************************************************************
// Convert 
// Takes 3 parameters: unsigned int sensedValue, which ranges from 0 - 1024
                       symbolic constant scaleValue
                       symbolic constant centerValue
// Returns singed int pulse width in ms.
****************************************************************************/
static unsigned int ServoConvert(unsigned int sensedValue, signed int scaleValue, signed int centerValue)
{
     unsigned int pulseWidth;
     pulseWidth= scaleValue * ((signed int)sensedValue - centerValue)/DENOMINATOR + CENTER_WIDTH;
     /*printf("\n\n\r %d \tsensedValue - centerValue", sensedValue - centerValue);
     printf("\n\r %d \tscaleValue * (sensedValue - centerValue)", scaleValue * (sensedValue - centerValue));
     printf("\n\r %d \tscaleValue * (sensedValue - centerValue)/DENOMINATOR", scaleValue * (sensedValue - centerValue)/DENOMINATOR);
     printf("\n\r %d \tpulseWidth", pulseWidth);
     */
     return pulseWidth; 
}


