/****************************************************************************
 Module
   U:\axwu\My Documents\ME218A Code\Lab4 Event Framework\MorseCodeDecoderSM\sources\MorseDecodeSM.c

 Revision
   1.0.1

 Description
   This is a template file for implementing state machines under the EF
   Event Framework.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 10/30/11 17:59 jec      fixed references to CurrentEvent in RunTemplateSM()
 10/23/11 18:20 jec      began conversion from SMTemplate.c (02/20/07 rev)
 11/07/11 12:00 Alice Wu adapted SMTemplate.c for MorseDecodeModule 
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "misc.h"
#include "EF_General.h"
#include "EF_Queue.h"
#include "EF_Events.h"
//#include "EF_SMTemplate.h"
#include "EF_Timers.h"
#include "EF_PostList.h"
#include "EventCheckers.h"
#include <ME218_C32.h>
#include <string.h>
#include <stdio.h>
#include "BoardScoreSM.h"
#include "TicTacDohSM.h"
//#include "InitializeIO.h"
#include "AnalogStuffSM.h"


/*----------------------------- Module Defines ----------------------------*/
#define QUEUE_SIZE 3

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/
//void ClearMorseChar(void);
//char DecodeMorse (void);

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match htat of enum in header file
static TicTacDohState_t CurrentState;
//static char MorseString[8];
static char LegalChars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890?.,:'-/()\"= !$&+;@_";
static char MorseCode[][8] ={ ".-","-...","-.-.","-..",".","..-.","--.",
                      "....","..",".---","-.-",".-..","--","-.","---",
                      ".--.","--.-",".-.","...","-","..-","...-",
                      ".--","-..-","-.--","--..",".----","..---",
                      "...--","....-",".....","-....","--...","---..",
                      "----.","-----","..--..",".-.-.-","--..--",
                      "---...",".----.","-....-","-..-.","-.--.-",
                      "-.--.-",".-..-.","-...-","-.-.--","...-..-",
                      ".-...",".-.-.","-.-.-.",".--.-.","..--.-"
                     };
static char ERR = '#';

// everybody needs a queue too!
EF_Event BoardScoreSMQueue[QUEUE_SIZE+1];


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitTemplateSM

 Parameters
     None

 Returns
     boolean, False if error in initialization, True otherwise

 Description
     Sets up the event queue and does any rother equired initialization for
     this state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 18:55
****************************************************************************/
boolean InitBoardScoreSM ( void )
{
  unsigned char ucResult1;
  EF_Event ThisEvent;
  
  ucResult1 = EF_InitQueue( BoardScoreSMQueue, ARRAY_SIZE(BoardScoreSMQueue) );
  if ( ucResult1 ==  QUEUE_SIZE ) // successful queue init
  {
    // put us into the Initial PseudoState
    CurrentState = InitBoardState;
    // post the initial transition event
    
    ThisEvent.EventType = EF_INIT;
    if (PostBoardScoreSM( ThisEvent ) == True)
    {
        puts("BoardScore Initialized \r\n");
        return True;
    }else
    {
        return False;
    }
    
  } else
  {
    return False;
  }
}

/****************************************************************************
 Function
     PostTemplateSM

 Parameters
     EF_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to the state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
****************************************************************************/
boolean PostBoardScoreSM( EF_Event ThisEvent )
{
  boolean bResult3;

  bResult3 = EF_EnQueueFIFO( BoardScoreSMQueue, ThisEvent );

  return bResult3;
}

/****************************************************************************
 Function
    RunTemplateSM

 Parameters
   void

 Returns
   boolean, Flase if an error encountered, True otherwise

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 10/23/11, 18:38
****************************************************************************/
boolean RunBoardScoreSM( void )
{
  EF_Event ThisEvent;

  // pull an event from the queue
  ThisEvent = EF_DeQueue( BoardScoreSMQueue );
  //puts("Running Board Score");

  // we should only call the Run function if we know that there is something in
  // the queue, so if we found EF_NO_EVENT something is wrong
  if (ThisEvent.EventType == EF_NO_EVENT )
  { 

    return False;
  }else
  {
    
    CurrentState = QueryTicTacDohSM();
    //puts("Board Score Current State  Queried from TicTacDoh \n\r");

    switch (CurrentState)
    {
        case WaitingToStart:       // If current state is initial Pseudo State
            {
                  puts("BoardScore in Waiting To Start");
                  return True;

            }
            break;
        
      case Player1Aiming:
         switch ( ThisEvent.EventType )
            {
                case EF_Button1Press : //If event is Button1Press
                {  EF_Event ThisEvent;
                  ThisEvent.EventType = EF_TargetHit;
                  ThisEvent.EventParam = 1;
                  EF_PostList00(ThisEvent);
                }
                break;
            }
            break;
            
      case Player2Aiming:
          switch ( ThisEvent.EventType )
            {
                case EF_Button2Press : //If event is Button2Press
                {  EF_Event ThisEvent;
                  ThisEvent.EventType = EF_TargetHit;
                  ThisEvent.EventParam = 1;
                  EF_PostList00(ThisEvent);
                }
                break;
            }
            break;
            
        // repeat state pattern as required for other states
      
       
    }  // end switch on Current State
    return True;
  }  //end else
}  //end function

 /****************************************************************************
 Function
     IsTemplateSMQEmpty

 Parameters
     void

 Returns
     boolean True if the Queue is empty, False if the queue has something in it

 Description
     tests if this state machine's queue is empty
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:28
****************************************************************************/
boolean IsBoardScoreSMQEmpty( void ){

  return EF_IsQueueEmpty( BoardScoreSMQueue );
}

/****************************************************************************
 Function
     QueryTemplateSM

 Parameters
     None

 Returns
     TicTacDohState_t The current state of the Template state machine

 Description
     returns the current state of the Template state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:21
****************************************************************************/
TicTacDohState_t QueryBoardScoreSM ( void )
{
   return(CurrentState);
}

/***************************************************************************
 private functions
 ***************************************************************************/
/*void ClearMorseChar (void)
{
*/
    /*
    int i;
    for (i = 0; i < 8; i++)
    {
      MorseString[i] = '\0';
    //strcpy(MorseString, "");
    }
    */
/*
    MorseString[0] = '\0';
}
*/
/*
char DecodeMorse (void) 
{
*/
   /*
   Takes no parameters, returns either a character or a symbolic value indicating 
   failure 
 
   For every entry in the array MorseCode 
   If MorseString is the same as current position in MorseCode  
       return contents of current position in LegalChars 
   EndIf 
   EndFor 
   return ERROR, since we didn't find a matching string in MorseCode 
   */
   /*
    unsigned char i;
    //
    for (i = 0; i < strlen(LegalChars); i++)
    {
      if(strcmp(MorseString,MorseCode[i]) == 0)
      {
         //printf("LegalChar: %c\r\n", LegalChars[i]);  
         return LegalChars[i];
      } //end if      
    } //end for
    return '#';
        
}
*/
void CheckTargetInput(void)
{
   //pulse register to get data
   //pulse shift clock to shift in data      
}


//#define Test
#ifdef Test
void main( void) {
    //InitializeIO(); //init some pins.
    
    //#define PhotoTransSC    BIT6HI   //T6  OUTPUT
    //#define PhotoTransRC    BIT7HI   //T7  OUTPUT  

}
#endif
