/****************************************************************************
 Module
   d:\me218\DevProjects\EventFramrwork\EF_SMTicTacDoh.c

 Revision
   1.0.1

 Description
   This is a TicTacDoh file for implementing state machines under the EF
   Event Framework.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 11/13/11 13:23 bdg      Setup for TicTacDoh
 11/07/11 11:26 jec      made the queue static
 10/30/11 17:59 jec      fixed references to CurrentEvent in TemplateSM()
 10/23/11 18:20 jec      began conversion from SMTemplate.c (02/20/07 rev)
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "misc.h"
#include "EF_General.h"
#include "EF_Queue.h"
#include "EF_Events.h"
#include "TicTacDohSM.h"
#include "stdio.h"

/*----------------------------- Module Defines ----------------------------*/
#define QUEUE_SIZE 3

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match htat of enum in header file
static TicTacDohState_t CurrentState;

// everybody needs a queue too!
static EF_Event SMQueue[QUEUE_SIZE+1];


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitTicTacDohSM

 Parameters
     None

 Returns
     boolean, False if error in initialization, True otherwise

 Description
     Sets up the event queue and does any rother equired initialization for
     this state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 18:55
****************************************************************************/
boolean InitTicTacDohSM ( void )
{
  unsigned char ucResult1;
  EF_Event ThisEvent;

  ucResult1 = EF_InitQueue( SMQueue, ARRAY_SIZE(SMQueue) );
  if ( ucResult1 ==  QUEUE_SIZE ) // successful queue init
  {
    // put us into the Initial PseudoState
    CurrentState = InitPState;
    // post the initial transition event
    ThisEvent.EventType = EF_INIT;
    if (PostTicTacDohSM( ThisEvent ) == True)
    {
        return True;
    }else
    {
        return False;
    }
  }else
  {
    return False;
  }
}

/****************************************************************************
 Function
     PostTicTacDohSM

 Parameters
     EF_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to the state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
****************************************************************************/
boolean PostTicTacDohSM( EF_Event ThisEvent )
{
  boolean bResult3;

  bResult3 = EF_EnQueueFIFO( SMQueue, ThisEvent );

  return bResult3;
}

/****************************************************************************
 Function
    RunTicTacDohSM

 Parameters
   void

 Returns
   boolean, Flase if an error encountered, True otherwise

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 10/23/11, 18:38
****************************************************************************/
boolean RunTicTacDohSM( void )
{
  EF_Event ThisEvent;

  // pull an event from the queue
  ThisEvent = EF_DeQueue( SMQueue );

  // we should only call the Run function if we know that there is something
  // in the queue, so if we found EF_NO_EVENT something is wrong
  if (ThisEvent.EventType == EF_NO_EVENT )
  {
    return False;
  }else
  {
    switch ( CurrentState )
    {
        case InitPState :// If current state is initial Psedudo State
            if ( ThisEvent.EventType == EF_INIT ) // only respond to EF_Init
            {
                // Put the machine into the actual initial state
                CurrentState = WaitingToStart;
                printf("\n\r STATE: WAITING TO START");
             } // end switch on CurrentEvent
             break;


        case WaitingToStart :// If current state is WaitingToStart
            
            switch ( ThisEvent.EventType )
            {
                case EF_Button1Press : //If event is Button1Press
                {
                    //The next state will be CoinToss
                    CurrentState = CoinToss;
                    printf("\n\r STATE: COIN TOSS"); 
                 }
                 break;
               case EF_Button2Press : //If event is event Button2Press
                {
                    //The next state will also be CoinToss
                    CurrentState = CoinToss;
                    printf("\n\r STATE: COIN TOSS");
                 }
                 break;            
            } // end switch on WaitingToStart
            break;


        case CoinToss :// If current state is CoinToss
            
            switch ( ThisEvent.EventType )
            {
                case EF_Player1WinsToss : 
                    CurrentState = Player1Aiming;
                    printf("\n\r STATE: PLAYER 1 AIMING");
                    break;
                    
                case EF_Player2WinsToss :
                    CurrentState = Player2Aiming;
                    printf("\n\r STATE: PLAYER 2 AIMING");
                    break;
          
            } // end switch on CoinToss
            break;

        case Player1Aiming :// If current state is Player1Aiming
            //printf("\n\r STATE: PLAYER 1 AIMING");
            switch ( ThisEvent.EventType )
            {
                case EF_TargetHit ://If event is EF_TargetHit  
                    printf("\t\t\t Target Hit!");
                    //Give player 2 aiming control
                    CurrentState = Player2Aiming;
                    printf("\n\r STATE: PLAYER 2 AIMING");
                    break;
                    
                case EF_Player1Timeout ://If event is EF_Player1Timeout  
                    printf("\t\t\t Timeout!");
                    // Dance!
                    CurrentState = VictoryDance;
                    printf("\n\r STATE: VICTORY DANCE");
                    break;                    

                case EF_WinEvent ://If event is EF_WinEvent  
                    printf("\t\t\t Win!");
                    // Dance!
                    CurrentState = VictoryDance;
                    printf("\n\r STATE: VICTORY DANCE");
                    break;                    
          
            } // end switch on Player1Aiming
            break;

        case Player2Aiming :// If current state is Player1Aiming
            //printf("\n\r STATE: PLAYER 2 AIMING");
            switch ( ThisEvent.EventType )
            {
                case EF_TargetHit ://If event is EF_TargetHit  
                    printf("\t\t\t Target Hit!");
                    //Give player 1 aiming control                    
                    CurrentState = Player1Aiming;
                    printf("\n\r STATE: PLAYER 1 AIMING");
                    break;
                    
                case EF_Player2Timeout ://If event is EF_Player2Timeout  
                    printf("\t\t\t Timeout!");
                    // Dance!
                    CurrentState = VictoryDance;
                    printf("\n\r STATE: VICTORY DANCE");
                    break;                    

                case EF_WinEvent ://If event is EF_WinEvent  
                    printf("\t\t\t Win!");
                    // Dance!
                    CurrentState = VictoryDance;
                    printf("\n\r STATE: VICTORY DANCE");
                    break;                    
          
            } // end switch on Player2Aiming
            break;


        case VictoryDance :// If current state is Player1Aiming
            //printf("\n\r STATE: VICTORY DANCE");
            switch ( ThisEvent.EventType )
            {
                case EF_VictoryDanceTimeout ://If event is EF_VictoryDanceTimeout  
                    printf("\t\t\t Done Dancing!");
                    //Reset for new game
                    CurrentState = WaitingToStart;
                    printf("\n\r STATE: WAITING TO START");
                    break;

            } // end switch on Player1Aiming
            break;



    } // end switch on Current State
    return True;
  }
}

 /****************************************************************************
 Function
     IsTicTacDohSMQEmpty

 Parameters
     void

 Returns
     boolean True if the Queue is empty, False if the queue has something in it

 Description
     tests if this state machine's queue is empty
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:28
****************************************************************************/
boolean IsTicTacDohSMQEmpty( void ){

  return EF_IsQueueEmpty( SMQueue );
}

/****************************************************************************
 Function
     QueryTicTacDohSM

 Parameters
     None

 Returns
     TicTacDohState_t The current state of the TicTacDoh state machine

 Description
     returns the current state of the TicTacDoh state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:21
****************************************************************************/
TicTacDohState_t QueryTicTacDohSM ( void )
{
   return(CurrentState);
}

/***************************************************************************
 private functions
 ***************************************************************************/

